
(* ******************************************************************** *)
(*
 * Project         : HOL-CSP - A Shallow Embedding of CSP in  Isabelle/HOL
 * Version         : 2.0
 *
 * Author          : Burkhart Wolff
 *                   (Based on HOL-CSP 1.0 by Haykal Tej and Burkhart Wolff)
 *
 * This file       : The Stop Process Definition
 *
 * Copyright (c) 2009 Universit Paris-Sud, France
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: ROOT.ML 8591 2009-09-26 09:58:14Z brucker $ *)

header{* The Stop Process Definition *}

theory     Stop
imports    Process Legacy
begin 

definition Stop :: "'\<alpha> process"  
where     "Stop \<equiv> Abs_Process ({(s, X). s = []}, {})"

lemma is_process_REP_Stop: "is_process ({(s, X). s = []},{})"
by(simp add: is_process_def FAILURES_def DIVERGENCES_def ftF_Nil)

lemma Rep_Abs_Stop : "Rep_Process (Abs_Process ({(s, X). s = []},{})) = ({(s, X). s = []},{})"
by(subst Abs_Process_inverse, simp add: Process_def is_process_REP_Stop, auto)

lemma F_Stop : "F Stop = {(s,X). s = []}"
by(simp add: Stop_def FAILURES_def F_def Rep_Abs_Stop)

lemma D_Stop: "D Stop = {}"
by(simp add: Stop_def DIVERGENCES_def D_def Rep_Abs_Stop)

lemma T_Stop: "T Stop = {[]}"
by(simp add: Stop_def TRACES_def FAILURES_def T_def Rep_Abs_Stop)

end

