(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: SimpleDMZInteger.thy 8881 2010-04-02 09:37:05Z brucker $ *)

theory 
  SimpleDMZInteger
imports 
  FWTesting
begin

profiling_on

text{*

This is the Simple DMZ example. There are three subnetworks, with
either none or some protocols allowed between then.

The used network model is the Integer model.
*}

constdefs
  intranet::"IntegerPort net"
  "intranet \<equiv> {{(a,b) . a = 3}}"

  dmz :: "IntegerPort net"
  "dmz \<equiv> {{(a,b). (a > 4) \<and> (a < 7)}}"

  internet :: "IntegerPort net"
  "internet \<equiv> {{(a,b).  \<not> (a=3 \<or> ((a > 4) \<and> (a < 7)))}}"



constdefs 
  Intranet_DMZ_Port :: "(IntegerPort,DummyContent) Policy"
  "Intranet_DMZ_Port \<equiv> (allow_from_ports_to {smtp,ftp} intranet dmz)"

  Intranet_Internet_Port :: "(IntegerPort,DummyContent) Policy"
  "Intranet_Internet_Port \<equiv> allow_all_from_port_to intranet http internet"
           
  Internet_DMZ_Port :: "(IntegerPort,DummyContent) Policy"
  "Internet_DMZ_Port \<equiv> (allow_from_ports_to {http,ftp} internet dmz)"

 
constdefs 
  policyPort :: "(IntegerPort, DummyContent) Policy"
 "policyPort \<equiv> deny_all ++ 
                  Intranet_Internet_Port ++
                  Intranet_DMZ_Port ++  
                  Internet_DMZ_Port"


lemmas PolicyLemmas =  dmz_def internet_def intranet_def
                       Intranet_Internet_Port_def Intranet_DMZ_Port_def 
                       Internet_DMZ_Port_def policyPort_def
                       src_def dest_def in_subnet_def
                       IntegerPortLemmas
                       content_def



constdefs
 not_in_same_net :: "(IntegerPort,DummyContent) packet \<Rightarrow> bool"
 "not_in_same_net x  \<equiv>  (src x \<sqsubset> internet \<longrightarrow> \<not> dest x \<sqsubset> internet) \<and> 
                     (src x \<sqsubset> intranet \<longrightarrow> \<not> dest x \<sqsubset> intranet) \<and> 
                     (src x \<sqsubset> dmz \<longrightarrow> \<not> dest x \<sqsubset> dmz)"

 


declare Ports [simp add]

profiling_on 

test_spec "not_in_same_net x \<longrightarrow> FUT x = policyPort x"
apply (prepare_fw_spec)
apply (simp add: not_in_same_net_def PolicyLemmas PortCombinators PolicyCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: PolicyLemmas not_in_same_net_def)
store_test_thm "PortPolicy"



testgen_params[iterations=100]

gen_test_data "PortPolicy"



thm PortPolicy.test_data

write_clocks "time_statsSimpleDMZInteger.tex"

profiling_off


end
