(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: ethz-datatype-base.thy 8881 2010-04-02 09:37:05Z brucker $ *)

theory 
  "ethz-datatype-base"
imports 
  FWTesting
begin

(* The subnetworks *)

datatype nets =  FWLInk |
 Any |
 I26|
 I27|
 I32|
 I4|
 Sip|
 Time|
 Dhcp|
 I4_32|
 I10_32|
 Eth_intern|
 Eth_private|
 Vpn_vpz
  
 
instance nets :: adr ..

types networks = "nets \<times> int"

constdefs
 FWLink :: "networks net"
"FWLink \<equiv> {{(a,b). a = FWLInk}}"

 any :: "networks net"
"any \<equiv> {{(a,b). a = Any}}"

 i26:: "networks net"
"i26 \<equiv> {{(a,b). a = I26}}"

 i27:: "networks net"
"i27 \<equiv> {{(a,b). a = I27 }}"

 i32:: "networks net"
"i32 \<equiv> {{(a,b). a =  I32}}"

 i4:: "networks net"
"i4 \<equiv> {{(a,b). a = I4 }}"

 sip:: "networks net"
"sip \<equiv> {{(a,b). a = Sip }}"

 time:: "networks net"
"time \<equiv> {{(a,b). a = Time}}"

 dhcp:: "networks net"
"dhcp \<equiv> {{(a,b). a =  Dhcp}}"
 
 i4_32:: "networks net"
"i4_32 \<equiv> {{(a,b). a =  I4_32}}"

 i10_32:: "networks net"
"i10_32 \<equiv> {{(a,b). a = I10_32}}"

 eth_intern:: "networks net"
"eth_intern \<equiv> {{(a,b). a = Eth_intern }}"
 
 eth_private:: "networks net"
"eth_private \<equiv> {{(a,b). a = Eth_private}}"

 vpn_vpz :: "networks net"
"vpn_vpz \<equiv> {{(a,b). a = Vpn_vpz}}"


defs (overloaded)
 src_port_def: "src_port (x::(networks,'b) packet) \<equiv> snd (src x)"
 dest_port_def: "dest_port (x::(networks,'b) packet) \<equiv> snd (dest x)"



constdefs
(* Mandatory: Global *)

 MG1 :: "(networks,DummyContent) Policy"
"MG1 \<equiv> allow_from_to_port (1::port) i26 any "

 MG2 :: "(networks,DummyContent) Policy"
"MG2 \<equiv> allow_from_to_ports {(1::port),(2::port),(4::port)} i27 any"

 MG3 :: "(networks,DummyContent) Policy"
"MG3 \<equiv> allow_from_port_to (1::port) any FWLink"

 MG4 :: "(networks,DummyContent) Policy"
"MG4 \<equiv> allow_from_to_port (3::port) FWLink FWLink
"
 MG5 :: "(networks,DummyContent) Policy"
"MG5 \<equiv> allow_from_to_port (5::port) i32 any"

 MG6 :: "(networks,DummyContent) Policy"
"MG6 \<equiv> allow_from_to_port (6::port) i32 i4 "
 
 MG7 :: "(networks,DummyContent) Policy"
"MG7 \<equiv> allow_from_to_ports {(6::port),(7::port)} FWLink i4"

 MG8 :: "(networks,DummyContent) Policy"
"MG8 \<equiv> allow_from_to_ports {(6::port),(7::port)} FWLink i4"

 MG9 :: "(networks,DummyContent) Policy"
"MG9 \<equiv> allow_from_to_port (5::port) any i32 "

 MG10 :: "(networks,DummyContent) Policy"
"MG10 \<equiv> allow_from_to_port (5::port) i32 any"

 MG11 :: "(networks,DummyContent) Policy"
"MG11 \<equiv> allow_from_to_port (1::port) FWLink any"

 MG12 :: "(networks,DummyContent) Policy"
"MG12 \<equiv> allow_from_to_port (3::port) FWLink FWLink"

 MG13 :: "(networks,DummyContent) Policy"
"MG13 \<equiv> allow_from_to_ports {(4::port), (2::port)} sip any "

(* Default Global *)

 DG1 :: "(networks,DummyContent) Policy"
"DG1 \<equiv> allow_from_to_port (8::port) time any "

 DG2:: "(networks,DummyContent) Policy"
"DG2 \<equiv> allow_from_to_port (1::port) dhcp any "

 DG3:: "(networks,DummyContent) Policy"
"DG3 \<equiv> deny_from_to_port (7::port) any any "

 


constdefs 
 MandatoryGlobalPolicy :: "(networks,DummyContent) Policy"
"MandatoryGlobalPolicy \<equiv> MG13 ++ MG12 ++  MG11 ++  MG10 ++  MG9 ++  MG8 ++  MG7 ++  MG6 ++ MG5 ++ MG4 ++ MG3 ++ MG2 ++ MG1"   
      
 MandatoryGlobalInsidePolicy :: "(networks,DummyContent) Policy"
"MandatoryGlobalInsidePolicy \<equiv> MG12 ++  MG11 ++  MG10 ++  MG9 ++  MG8"
         
 MandatoryGlobalOutsidePolicy :: "(networks,DummyContent) Policy"
"MandatoryGlobalOutsidePolicy \<equiv>   MG13 ++ MG7 ++ MG6 ++ MG5 ++ MG4 ++ MG3 ++ MG2 ++ MG1"       

 DefaultGlobalPolicy :: "(networks,DummyContent) Policy"
"DefaultGlobalPolicy \<equiv> DG3 ++ DG2 ++ DG1"  

 lemmas PolicyLemmas =  
 FWLink_def
 any_def
 i26_def
 i27_def
 i32_def
 i4_def
 sip_def
 time_def
 dhcp_def
 i4_32_def
 i10_32_def
 eth_intern_def
 eth_private_def
 vpn_vpz_def
 MG1_def MG2_def MG3_def MG4_def MG5_def MG6_def MG7_def MG8_def MG9_def MG10_def MG11_def MG12_def MG13_def
 DG1_def DG2_def DG3_def
 MandatoryGlobalPolicy_def MandatoryGlobalOutsidePolicy_def MandatoryGlobalInsidePolicy_def DefaultGlobalPolicy_def
 src_def dest_def in_subnet_def dest_port_def

constdefs 
 not_in_same_net :: "(networks,DummyContent) packet \<Rightarrow> bool"
"not_in_same_net x \<equiv>  ((src x \<noteq> dest x))"                   

test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ DefaultGlobalPolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas) 
store_test_thm "DG"


test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalInsidePolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas) 
store_test_thm "MGI"


test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalOutsidePolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas) 
store_test_thm "MGO"


test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalPolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PortCombinators PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas) 
store_test_thm "MG"



test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ DefaultGlobalPolicy ++ MandatoryGLobalPolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas  PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas) 
store_test_thm "all"



end
